# -*- coding: utf-8 -*-

"""Misc utilities for xml_delta."""

import logging
import logging.handlers
import os
import time

logger = logging.getLogger('detailed')

PER_ROW = 10


class EnhancedRotatingFileHandler(logging.handlers.TimedRotatingFileHandler):
    """Custom rotating file handler which is both sized and timed.

    Logs to a set of files, and switches from one file to the next when the
    current file reaches a certain size, or at certain timed intervals,
    whichever happens first.
    """

    # Based on https://stackoverflow.com/a/6347764

    def __init__(
        self,
        filename,
        when='h',
        interval=1,
        backupCount=0,
        encoding=None,
        delay=False,
        utc=False,
        maxBytes=0
    ):
        """
        This is just a combination of TimedRotatingFileHandler and
        RotatingFileHandler (adds maxBytes to TimedRotatingFileHandler).
        """
        logging.handlers.TimedRotatingFileHandler.__init__(
            self, filename, when, interval, backupCount, encoding, delay, utc
        )
        self.maxBytes = maxBytes

    def shouldRollover(self, record):
        """Determine if rollover should occur.

        Basically, see if the supplied record would cause the file to exceed
        the size limit we have, and if not compare times.
        """
        if self.stream is None:  # delay was set...
            self.stream = self._open()
        if self.maxBytes > 0:  # are we rolling over?
            msg = "%s\n" % self.format(record)
            self.stream.seek(0, 2)  # due to non-posix-compliant Windows feature
            if self.stream.tell() + len(msg) >= self.maxBytes:
                return 1
        t = int(time.time())
        if t >= self.rolloverAt:
            return 1
        return 0

    def doRollover(self):
        """
        Do a rollover; in this case, a date/time stamp is appended to the
        filename when the rollover happens.  However, you want the file to be
        named for the start of the interval, not the current time.  If there is
        a backup count, then we have to get a list of matching filenames, sort
        them and remove the one with the oldest suffix.
        """
        if self.stream:
            self.stream.close()
        # get the time that this sequence started at and make it a TimeTuple
        currentTime = int(time.time())
        dstNow = time.localtime(currentTime)[-1]
        t = self.rolloverAt - self.interval
        if self.utc:
            timeTuple = time.gmtime(t)
        else:
            timeTuple = time.localtime(t)
            dstThen = timeTuple[-1]
            if dstNow != dstThen:
                if dstNow:
                    addend = 3600
                else:
                    addend = -3600
                timeTuple = time.localtime(t + addend)
        dfn = self.baseFilename + "." + time.strftime(self.suffix, timeTuple)
        if self.backupCount > 0:
            cnt = 1
            dfn2 = "%s.%03d" % (dfn, cnt)
            while os.path.exists(dfn2):
                dfn2 = "%s.%03d" % (dfn, cnt)
                cnt += 1
            os.rename(self.baseFilename, dfn2)
            for s in self.getFilesToDelete():
                os.remove(s)
        else:
            if os.path.exists(dfn):
                os.remove(dfn)
            os.rename(self.baseFilename, dfn)
        self.mode = 'w'
        self.stream = self._open()
        newRolloverAt = self.computeRollover(currentTime)
        while newRolloverAt <= currentTime:
            newRolloverAt = newRolloverAt + self.interval
        # If DST changes and midnight or weekly rollover, adjust for this.
        if (self.when == 'MIDNIGHT' or self.when.startswith('W')) and not self.utc:
            dstAtRollover = time.localtime(newRolloverAt)[-1]
            if dstNow != dstAtRollover:
                if not dstNow:  # DST kicks in before next rollover, so we need to deduct an hour
                    addend = -3600
                else:  # DST bows out before next rollover, so we need to add an hour
                    addend = 3600
                newRolloverAt += addend
        self.rolloverAt = newRolloverAt

    def getFilesToDelete(self):
        """Determine the files to delete when rolling over."""
        dirName, baseName = os.path.split(self.baseFilename)
        fileNames = os.listdir(dirName)
        result = []
        prefix = baseName + "."
        plen = len(prefix)
        for fileName in fileNames:
            if fileName[:plen] == prefix:
                suffix = fileName[plen:-4]
                if self.extMatch.match(suffix):
                    result.append(os.path.join(dirName, fileName))
        result.sort()
        if len(result) < self.backupCount:
            result = []
        else:
            result = result[:len(result) - self.backupCount]
        return result


def log(listings, msg):
    tmp = []
    i = 0
    for listing in listings:
        tmp.append(listing.storage_key)
        i += 1
        if i == PER_ROW:
            logger.info('%s %s', msg, ' '.join(tmp))
            tmp = []
            i = 0
    if tmp:
        logger.info('%s %s', msg, ' '.join(tmp))
